/* unixserial.c: tty line interface code for Pilot serial comms under UNIX
 *
 * Copyright (c) 1996, 1997, D. Jeff Dionne & Kenneth Albanowski.
 * This is free software, licensed under the GNU Public License V2.
 * See the file COPYING for details.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include "pi-source.h"
#include "pi-socket.h"
#include "pi-serial.h"
#include "pi-slp.h"
#include "pi-syspkt.h"
#include "pi-padp.h"
#include <qdos.h>
#include <sys/select.h>

static int s_changebaud(struct pi_socket *ps);
static int s_close(struct pi_socket *ps);
static int s_write(struct pi_socket *ps);
static int s_read(struct pi_socket *ps, int timeout);

int pi_serial_open(struct pi_socket *ps, struct pi_sockaddr * addr,
                   int addrlen)
{
  char * tty = addr->pi_device;

  int i;

  if ((!tty) || !strlen(tty))
    tty = getenv("PILOTPORT");
  if (!tty)
    tty = "<Null>";
  
  if ((ps->mac->fd = open(tty, O_RDWR | O_NONBLOCK )) == -1) {
    return -1;     /* errno already set */
  }
#if 0
  if (!isatty(ps->mac->fd)) {
    close(ps->mac->fd);
    errno = EINVAL;
    return -1;
  }
#endif

  if (ps->sd) {
    int orig = ps->mac->fd;
    ps->mac->fd = dup2(ps->mac->fd, ps->sd);
    if (ps->mac->fd != orig)
      close(orig);
  }

  ps->serial_close = s_close;
  ps->serial_read = s_read;
  ps->serial_write = s_write;
  ps->serial_changebaud = s_changebaud;
  return ps->mac->fd;
}


static int s_changebaud(struct pi_socket *ps)
{

  return 0;
}



static int s_close(struct pi_socket *ps)
{
  int result;
  result = close(ps->mac->fd);
  ps->mac->fd = 0;

  return result;
}

static int s_write(struct pi_socket *ps)
{
  struct pi_skb *skb;
  int nwrote, len;
  int i;

  if (ps->txq) {
    ps->busy++;

    skb = ps->txq;
    ps->txq = skb->next;

    len = 0;
    while (len<skb->len) {
      nwrote = 0;
      nwrote=write(ps->mac->fd,skb->data,skb->len);
      if (nwrote<=0)
        break; /* transmission failure */
      len += nwrote;
    }
    ps->tx_bytes += skb->len;
    free(skb);

    ps->busy--;
    
    return 1;
  }
  return 0;
}

static int s_read(struct pi_socket *ps, int timeout)
{
  int r;
  unsigned char *buf;
  int qlch;
  
  qlch = getchid(ps->mac->fd);
  if(timeout == 0)
  {
      timeout = -1;
  }
  else
  {
      timeout *= 5;
  }
  
  pi_serial_flush(ps);              /* We likely want to be in sync with tx */
  if (!ps->mac->expect) slp_rx(ps);  /* let SLP know we want a packet */

  while (ps->mac->expect)
  {
    buf = ps->mac->buf;

    while (ps->mac->expect)
    {
        r = io_fstrg(qlch, timeout, buf, ps->mac->expect);
        if(r <= 0)
        {
                /* otherwise throw out any current packet and return */
#ifdef DEBUG
            fprintf(stderr, "Serial RX: timeout\n");
#endif
            ps->mac->state = ps->mac->expect = 1;
            ps->mac->buf = ps->mac->rxb->data;
            ps->rx_errors++;
            return 0;
        }
        ps->rx_bytes += r;
        buf += r;
        ps->mac->expect -= r;
    }
    slp_rx(ps);
  }
  return 0;
}



